@@ -5,6 +5,7 @@
 #include "spirv/unified1/spirv.hpp"
 #include "spirv_internal.hpp"
 #include "llvm/BinaryFormat/Dwarf.h"
+#include "llvm/IR/DebugInfoMetadata.h"
 
 namespace SPIRVDebug {
 
@@ -55,6 +56,13 @@ enum Instruction {
   Source                        = 35,
   ModuleINTEL                   = 36,
   InstCount                     = 37,
+  FunctionDefinition            = 101,
+  SourceContinued               = 102,
+  DebugLine                     = 103,
+  DebugNoLine                   = 104,
+  BuildIdentifier               = 105,
+  StoragePath                   = 106,
+  EntryPoint                    = 107,
   Module                        = 200,
   TypeSubrange                  = 201,
   TypeArrayDynamic              = 202,
@@ -62,25 +70,27 @@ enum Instruction {
 };
 
 enum Flag {
-  FlagIsProtected         = 1 << 0,
-  FlagIsPrivate           = 1 << 1,
-  FlagIsPublic            = FlagIsPrivate | FlagIsProtected,
-  FlagAccess              = FlagIsPublic,
-  FlagIsLocal             = 1 << 2,
-  FlagIsDefinition        = 1 << 3,
-  FlagIsFwdDecl           = 1 << 4,
-  FlagIsArtificial        = 1 << 5,
-  FlagIsExplicit          = 1 << 6,
-  FlagIsPrototyped        = 1 << 7,
-  FlagIsObjectPointer     = 1 << 8,
-  FlagIsStaticMember      = 1 << 9,
-  FlagIsIndirectVariable  = 1 << 10,
-  FlagIsLValueReference   = 1 << 11,
-  FlagIsRValueReference   = 1 << 12,
-  FlagIsOptimized         = 1 << 13,
-  FlagIsEnumClass         = 1 << 14,
-  FlagTypePassByValue     = 1 << 15,
-  FlagTypePassByReference = 1 << 16,
+  FlagIsProtected           = 1 << 0,
+  FlagIsPrivate             = 1 << 1,
+  FlagIsPublic              = FlagIsPrivate | FlagIsProtected,
+  FlagAccess                = FlagIsPublic,
+  FlagIsLocal               = 1 << 2,
+  FlagIsDefinition          = 1 << 3,
+  FlagIsFwdDecl             = 1 << 4,
+  FlagIsArtificial          = 1 << 5,
+  FlagIsExplicit            = 1 << 6,
+  FlagIsPrototyped          = 1 << 7,
+  FlagIsObjectPointer       = 1 << 8,
+  FlagIsStaticMember        = 1 << 9,
+  FlagIsIndirectVariable    = 1 << 10,
+  FlagIsLValueReference     = 1 << 11,
+  FlagIsRValueReference     = 1 << 12,
+  FlagIsOptimized           = 1 << 13,
+  FlagIsEnumClass           = 1 << 14,
+  FlagTypePassByValue       = 1 << 15,
+  FlagTypePassByReference   = 1 << 16,
+  FlagUnknownPhysicalLayout = 1 << 17,
+  FlagBitField              = 1 << 18
 };
 
 enum EncodingTag {
@@ -91,7 +101,8 @@ enum EncodingTag {
   Signed       = 4,
   SignedChar   = 5,
   Unsigned     = 6,
-  UnsignedChar = 7
+  UnsignedChar = 7,
+  Complex      = 8
 };
 
 enum CompositeTypeTag {
@@ -257,23 +268,25 @@ enum ExpressionOpCode {
   DerefSize  = 146,
   XderefSize = 147,
   Nop        = 148,
-  PushObjectAddress = 149,
-  Call2             = 150,
-  Call4             = 151,
-  CallRef           = 152,
-  FormTlsAddress    = 153,
-  CallFrameCfa      = 154,
-  ImplicitValue     = 155,
-  ImplicitPointer   = 156,
-  Addrx             = 157,
-  Constx            = 158,
-  EntryValue        = 159,
-  ConstTypeOp       = 160,
-  RegvalType        = 161,
-  DerefType         = 162,
-  XderefType        = 163,
-  Reinterpret       = 164,
-  LLVMArg           = 165,
+  PushObjectAddress  = 149,
+  Call2              = 150,
+  Call4              = 151,
+  CallRef            = 152,
+  FormTlsAddress     = 153,
+  CallFrameCfa       = 154,
+  ImplicitValue      = 155,
+  ImplicitPointer    = 156,
+  Addrx              = 157,
+  Constx             = 158,
+  EntryValue         = 159,
+  ConstTypeOp        = 160,
+  RegvalType         = 161,
+  DerefType          = 162,
+  XderefType         = 163,
+  Reinterpret        = 164,
+  LLVMArg            = 165,
+  ImplicitPointerTag = 166,
+  TagOffset          = 167,
 };
 
 enum ImportedEntityTag {
@@ -281,6 +294,12 @@ enum ImportedEntityTag {
   ImportedDeclaration = 1,
 };
 
+enum FileChecksumKind {
+  MD5 = 0,
+  SHA1 = 1,
+  SHA256 = 2,
+};
+
 namespace Operand {
 
 namespace CompilationUnit {
@@ -289,24 +308,49 @@ enum {
   DWARFVersionIdx          = 1,
   SourceIdx                = 2,
   LanguageIdx              = 3,
-  OperandCount             = 4
+  // For NonSemantic.Shader.DebugInfo.200
+  ProducerIdx              = 4,
+  MinOperandCount          = 4
 };
 }
 
 namespace Source {
 enum {
-  FileIdx      = 0,
-  TextIdx      = 1,
-  OperandCount = 2
+  FileIdx         = 0,
+  TextIdx         = 1,
+  // For NonSemantic.Shader.DebugInfo.200
+  ChecksumKind    = 1,
+  ChecksumValue   = 2,
+  TextNonSemIdx   = 3,
+  MinOperandCount = 1,
+  MaxOperandCount = 4
+};
+}
+
+namespace BuildIdentifier {
+enum {
+  IdentifierIdx = 0,
+  FlagsIdx      = 1,
+  OperandCount  = 2
+};
+}
+
+namespace StoragePath {
+enum {
+  PathIdx       = 0,
+  OperandCount  = 1
 };
 }
 
 namespace TypeBasic {
 enum {
-  NameIdx      = 0,
-  SizeIdx      = 1,
-  EncodingIdx  = 2,
-  OperandCount = 3
+  NameIdx                 = 0,
+  SizeIdx                 = 1,
+  EncodingIdx             = 2,
+  // For NonSemantic Specs
+  FlagsIdx                = 3,
+  OperandCountOCL         = 3,
+  OperandCountNonSemantic = 4
 };
 }
 
@@ -352,11 +396,12 @@ namespace TypeVector = TypeArray;
 
 namespace TypeSubrange {
 enum {
-  CountIdx        = 0,
-  LowerBoundIdx   = 1,
-  UpperBoundIdx   = 2,
+  LowerBoundIdx   = 0,
+  UpperBoundIdx   = 1,
+  CountIdx        = 2,
   StrideIdx       = 3,
-  OperandCount    = 4
+  MinOperandCount = 3,
+  MaxOperandCount = 4
 };
 }
 
@@ -425,6 +470,7 @@ enum {
 }
 
 namespace TypeMember {
+namespace OpenCL {
 enum {
   NameIdx         = 0,
   TypeIdx         = 1,
@@ -440,6 +486,22 @@ enum {
 };
 }
 
+namespace NonSemantic {
+enum {
+  NameIdx         = 0,
+  TypeIdx         = 1,
+  SourceIdx       = 2,
+  LineIdx         = 3,
+  ColumnIdx       = 4,
+  OffsetIdx       = 5,
+  SizeIdx         = 6,
+  FlagsIdx        = 7,
+  ValueIdx        = 8,
+  MinOperandCount = 8
+};
+}
+} // namespace TypeMember
+
 namespace TypeInheritance {
 namespace NonSemantic {
 enum {
@@ -545,30 +607,79 @@ enum {
 
 namespace Function {
 enum {
-  NameIdx         = 0,
-  TypeIdx         = 1,
-  SourceIdx       = 2,
-  LineIdx         = 3,
-  ColumnIdx       = 4,
-  ParentIdx       = 5,
-  LinkageNameIdx  = 6,
-  FlagsIdx        = 7,
-  ScopeLineIdx    = 8,
-  FunctionIdIdx   = 9,
-  DeclarationIdx  = 10,
-  TargetFunctionNameIdx  = 11,
-  MinOperandCount = 10
+  NameIdx               = 0,
+  TypeIdx               = 1,
+  SourceIdx             = 2,
+  LineIdx               = 3,
+  ColumnIdx             = 4,
+  ParentIdx             = 5,
+  LinkageNameIdx        = 6,
+  FlagsIdx              = 7,
+  ScopeLineIdx          = 8,
+  FunctionIdIdx         = 9,
+  DeclarationIdx        = 10,
+  MinOperandCount       = 10,
+
+// Only for NonSemantic.Shader.DebugInfo.*
+// No operand FunctionId
+  DeclarationNonSemIdx  = 9,
+  MinOperandCountNonSem = 9,
+  // Only for NonSemantic.Shader.DebugInfo.200
+  TargetFunctionNameIdx = 10,
+};
+}
+
+namespace FunctionDefinition {
+enum {
+  FunctionIdx     = 0,
+  DefinitionIdx   = 1,
+  OperandCount    = 2
+};
+}
+
+namespace SourceContinued {
+enum {
+  TextIdx      = 0,
+  OperandCount = 1
+};
+}
+
+namespace DebugLine {
+enum {
+  SourceIdx      = 0,
+  StartIdx       = 1,
+  EndIdx         = 2,
+  ColumnStartIdx = 3,
+  ColumnEndIdx   = 4,
+  OperandCount   = 5
+};
+}
+
+namespace DebugNoLine {
+enum {
+  OperandCount = 0
+};
+}
+
+namespace EntryPoint {
+enum {
+  EntryPointIdx        = 0,
+  CompilationUnitIdx   = 1,
+  CompilerSignatureIdx = 2,
+  CommandLineArgsIdx   = 3,
+  OperandCount         = 4
 };
 }
 
 namespace LexicalBlock {
 enum {
-  SourceIdx       = 0,
-  LineIdx         = 1,
-  ColumnIdx       = 2,
-  ParentIdx       = 3,
-  NameIdx         = 4,
-  MinOperandCount = 4
+  SourceIdx          = 0,
+  LineIdx            = 1,
+  ColumnIdx          = 2,
+  ParentIdx          = 3,
+  MinOperandCount    = 4,
+  NameIdx            = 4,
+  InlineNamespaceIdx = 5,
 };
 }
 
@@ -594,13 +705,25 @@ namespace NoScope {
 }
 
 namespace InlinedAt {
+namespace OpenCL {
 enum {
   LineIdx         = 0,
   ScopeIdx        = 1,
   InlinedIdx      = 2,
   MinOperandCount = 2
 };
-}
+} // namespace OpenCL
+
+namespace NonSemantic {
+enum {
+  LineIdx         = 0,
+  ColumnIdx       = 1,
+  ScopeIdx        = 2,
+  InlinedIdx      = 3,
+  MinOperandCount = 3
+};
+} // namespace NonSemantic
+} // namespace ImportedEntity
 
 namespace LocalVariable {
 enum {
@@ -659,15 +782,15 @@ static std::map<ExpressionOpCode, unsigned> OpCountMap {
   { Constu,             2 },
   { Fragment,           3 },
   { Convert,            3 },
-  // { Addr,               2 }, /* not implemented */
-  // { Const1u,            2 },
-  // { Const1s,            2 },
-  // { Const2u,            2 },
-  // { Const2s,            2 },
-  // { Const4u,            2 },
-  // { Const4s,            2 },
-  // { Const8u,            2 },
-  // { Const8s,            2 },
+  { Addr,               2 },
+  { Const1u,            2 },
+  { Const1s,            2 },
+  { Const2u,            2 },
+  { Const2s,            2 },
+  { Const4u,            2 },
+  { Const4s,            2 },
+  { Const8u,            2 },
+  { Const8s,            2 },
   { Consts,             2 },
   { Dup,                1 },
   { Drop,               1 },
@@ -686,14 +809,14 @@ static std::map<ExpressionOpCode, unsigned> OpCountMap {
   { Shr,                1 },
   { Shra,               1 },
   { Xor,                1 },
-  // { Bra,                2 }, /* not implemented */
+  { Bra,                2 },
   { Eq,                 1 },
   { Ge,                 1 },
   { Gt,                 1 },
   { Le,                 1 },
   { Lt,                 1 },
   { Ne,                 1 },
-  // { Skip,               2 }, /* not implemented */
+  { Skip,               2 },
   { Lit0,               1 },
   { Lit1,               1 },
   { Lit2,               1 },
@@ -791,33 +914,38 @@ static std::map<ExpressionOpCode, unsigned> OpCountMap {
   { Breg30,             2 },
   { Breg31,             2 },
   { Regx,               2 },
-  // { Fbreg,              1 }, /* not implemented */
+  { Fbreg,              1 },
   { Bregx,              3 },
-  // { Piece,              2 }, /* not implemented */
+  { Piece,              2 },
   { DerefSize,          2 },
   { XderefSize,         2 },
   { Nop,                1 },
   { PushObjectAddress,  1 },
-  // { Call2,              2 }, /* not implemented */
-  // { Call4,              2 },
-  // { CallRef,            2 },
-  // { FormTlsAddress,     1 },
-  // { CallFrameCfa,       1 },
-  // { ImplicitValue,      3 },
-  // { ImplicitPointer,    3 },
-  // { Addrx,              2 },
-  // { Constx,             2 },
-  // { EntryValue,         3 },
-  // { ConstTypeOp,        4 },
-  // { RegvalType,         3 },
-  // { DerefType,          3 },
-  // { XderefType,         3 },
-  // { Reinterpret,        2 },
+  { Call2,              2 },
+  { Call4,              2 },
+  { CallRef,            2 },
+  { FormTlsAddress,     1 },
+  { CallFrameCfa,       1 },
+  { ImplicitValue,      3 },
+  { ImplicitPointer,    3 },
+  { Addrx,              2 },
+  { Constx,             2 },
+  { EntryValue,         3 },
+  { ConstTypeOp,        4 },
+  { RegvalType,         3 },
+  { DerefType,          3 },
+  { XderefType,         3 },
+  { Reinterpret,        2 },
   { LLVMArg,            2 },
+  { ImplicitPointerTag, 2 },
+  { TagOffset,          2 },
 };
 }
 
 namespace ImportedEntity {
+namespace OpenCL {
+// it's bugged version, note 2nd index is missing
+// FIXME: need to remove it after some graceful period
 enum {
   NameIdx      = 0,
   TagIdx       = 1,
@@ -828,7 +956,20 @@ enum {
   ParentIdx    = 7,
   OperandCount = 8
 };
-}
+} // namespace OpenCL
+namespace NonSemantic {
+enum {
+  NameIdx      = 0,
+  TagIdx       = 1,
+  SourceIdx    = 2,
+  EntityIdx    = 3,
+  LineIdx      = 4,
+  ColumnIdx    = 5,
+  ParentIdx    = 6,
+  OperandCount = 7
+};
+} // namespace NonSemantic
+} // namespace ImportedEntity
 
 namespace ModuleINTEL {
 enum {
@@ -846,8 +987,9 @@ enum {
 
 // helper function to get parent scope of debug instruction, to be used
 // to determine with which compile unit the particular instruction relates
-inline bool hasDbgInstParentScopeIdx(const uint32_t Kind,
-                                     uint32_t &ParentScopeIdx) {
+inline bool hasDbgInstParentScopeIdx(
+    const uint32_t Kind, uint32_t &ParentScopeIdx,
+    const SPIRV::SPIRVExtInstSetKind ExtKind = SPIRV::SPIRVEIS_OpenCL) {
   switch (Kind) {
   case SPIRVDebug::Typedef:
     ParentScopeIdx = Typedef::ParentIdx;
@@ -856,10 +998,10 @@ inline bool hasDbgInstParentScopeIdx(const uint32_t Kind,
     ParentScopeIdx = TypeEnum::ParentIdx;
     return true;
   case SPIRVDebug::TypeComposite:
-    ParentScopeIdx = TypeMember::ParentIdx;
+    ParentScopeIdx = TypeMember::OpenCL::ParentIdx;
     return true;
   case SPIRVDebug::TypeInheritance:
-    if (Kind == SPIRV::SPIRVEIS_OpenCL_DebugInfo_100)
+    if (ExtKind == SPIRV::SPIRVEIS_OpenCL_DebugInfo_100)
       ParentScopeIdx = TypeInheritance::OpenCL::ParentIdx;
     else
       ParentScopeIdx = TypeInheritance::NonSemantic::ParentIdx;
@@ -870,6 +1012,9 @@ inline bool hasDbgInstParentScopeIdx(const uint32_t Kind,
   case SPIRVDebug::Function:
     ParentScopeIdx = Function::ParentIdx;
     return true;
+  case SPIRVDebug::EntryPoint:
+    ParentScopeIdx = EntryPoint::CompilationUnitIdx;
+    return true;
   case SPIRVDebug::LexicalBlock:
     ParentScopeIdx = LexicalBlock::ParentIdx;
     return true;
@@ -880,13 +1025,19 @@ inline bool hasDbgInstParentScopeIdx(const uint32_t Kind,
     ParentScopeIdx = Scope::ScopeIdx;
     return true;
   case SPIRVDebug::InlinedAt:
-    ParentScopeIdx = InlinedAt::ScopeIdx;
+    if (ExtKind == SPIRV::SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
+      ParentScopeIdx = InlinedAt::NonSemantic::ScopeIdx;
+    else
+      ParentScopeIdx = InlinedAt::OpenCL::ScopeIdx;
     return true;
   case SPIRVDebug::LocalVariable:
     ParentScopeIdx = LocalVariable::ParentIdx;
     return true;
   case SPIRVDebug::ImportedEntity:
-    ParentScopeIdx = ImportedEntity::ParentIdx;
+    if (ExtKind == SPIRV::SPIRVEIS_OpenCL_DebugInfo_100)
+      ParentScopeIdx = ImportedEntity::OpenCL::ParentIdx;
+    else
+      ParentScopeIdx = ImportedEntity::NonSemantic::ParentIdx;
     return true;
   case SPIRVDebug::ModuleINTEL:
     ParentScopeIdx = ModuleINTEL::ParentIdx;
@@ -908,6 +1059,7 @@ inline spv::SourceLanguage convertDWARFSourceLangToSPIRV(dwarf::SourceLanguage D
   switch (DwarfLang) {
   // When updating this function, make sure to also
   // update convertSPIRVSourceLangToDWARF()
+  case dwarf::SourceLanguage::DW_LANG_C_plus_plus_17:
   case dwarf::SourceLanguage::DW_LANG_C_plus_plus_14:
   case dwarf::SourceLanguage::DW_LANG_C_plus_plus:
     return spv::SourceLanguage::SourceLanguageCPP_for_OpenCL;
@@ -943,7 +1095,7 @@ inline dwarf::SourceLanguage convertSPIRVSourceLangToDWARF(unsigned SourceLang)
   case spv::SourceLanguage::SourceLanguageOpenCL_CPP:
     return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
   case spv::SourceLanguage::SourceLanguageCPP_for_OpenCL:
-    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_17;
   case spv::SourceLanguage::SourceLanguageOpenCL_C:
   case spv::SourceLanguage::SourceLanguageESSL:
   case spv::SourceLanguage::SourceLanguageGLSL:
@@ -962,6 +1114,10 @@ inline spv::SourceLanguage convertDWARFSourceLangToSPIRVNonSemanticDbgInfo(
   case dwarf::SourceLanguage::DW_LANG_OpenCL:
     return spv::SourceLanguage::SourceLanguageOpenCL_C;
 
+  case dwarf::SourceLanguage::DW_LANG_C_plus_plus_20:
+    return spv::internal::SourceLanguageCPP20;
+  case dwarf::SourceLanguage::DW_LANG_C_plus_plus_17:
+    return spv::internal::SourceLanguageCPP17;
   case dwarf::SourceLanguage::DW_LANG_C_plus_plus_14:
     return spv::internal::SourceLanguageCPP14;
   case dwarf::SourceLanguage::DW_LANG_C_plus_plus_11:
@@ -977,6 +1133,8 @@ inline spv::SourceLanguage convertDWARFSourceLangToSPIRVNonSemanticDbgInfo(
     return spv::internal::SourceLanguageC99;
   case dwarf::SourceLanguage::DW_LANG_C11:
     return spv::internal::SourceLanguageC11;
+  case dwarf::SourceLanguage::DW_LANG_C17:
+    return spv::internal::SourceLanguageC17;
 
   case dwarf::SourceLanguage::DW_LANG_Python:
     return spv::internal::SourceLanguagePython;
@@ -997,6 +1155,8 @@ inline spv::SourceLanguage convertDWARFSourceLangToSPIRVNonSemanticDbgInfo(
     return spv::internal::SourceLanguageFortran2003;
   case dwarf::SourceLanguage::DW_LANG_Fortran08:
     return spv::internal::SourceLanguageFortran2008;
+  case dwarf::SourceLanguage::DW_LANG_Fortran18:
+    return spv::internal::SourceLanguageFortran2018;
   default:
     return spv::SourceLanguage::SourceLanguageUnknown;
   }
@@ -1010,8 +1170,12 @@ convertSPIRVSourceLangToDWARFNonSemanticDbgInfo(unsigned SourceLang) {
   case spv::SourceLanguage::SourceLanguageOpenCL_CPP:
     return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
   case spv::SourceLanguage::SourceLanguageCPP_for_OpenCL:
-    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_17;
 
+  case spv::internal::SourceLanguageCPP20:
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_20;
+  case spv::internal::SourceLanguageCPP17:
+    return dwarf::SourceLanguage::DW_LANG_C_plus_plus_17;
   case spv::internal::SourceLanguageCPP14:
     return dwarf::SourceLanguage::DW_LANG_C_plus_plus_14;
   case spv::internal::SourceLanguageCPP11:
@@ -1027,6 +1191,8 @@ convertSPIRVSourceLangToDWARFNonSemanticDbgInfo(unsigned SourceLang) {
     return dwarf::SourceLanguage::DW_LANG_C99;
   case spv::internal::SourceLanguageC11:
     return dwarf::SourceLanguage::DW_LANG_C11;
+  case spv::internal::SourceLanguageC17:
+    return dwarf::SourceLanguage::DW_LANG_C17;
 
   case spv::internal::SourceLanguagePython:
     return dwarf::SourceLanguage::DW_LANG_Python;
@@ -1047,6 +1213,8 @@ convertSPIRVSourceLangToDWARFNonSemanticDbgInfo(unsigned SourceLang) {
     return dwarf::SourceLanguage::DW_LANG_Fortran03;
   case spv::internal::SourceLanguageFortran2008:
     return dwarf::SourceLanguage::DW_LANG_Fortran08;
+  case spv::internal::SourceLanguageFortran2018:
+    return dwarf::SourceLanguage::DW_LANG_Fortran18;
 
   case spv::SourceLanguage::SourceLanguageOpenCL_C:
   case spv::SourceLanguage::SourceLanguageESSL:
@@ -1070,6 +1238,7 @@ inline void DbgEncodingMap::init() {
   add(dwarf::DW_ATE_signed_char,       SPIRVDebug::SignedChar);
   add(dwarf::DW_ATE_unsigned,          SPIRVDebug::Unsigned);
   add(dwarf::DW_ATE_unsigned_char,     SPIRVDebug::UnsignedChar);
+  add(dwarf::DW_ATE_complex_float,     SPIRVDebug::Complex);
 }
 
 typedef SPIRVMap<dwarf::Tag, SPIRVDebug::TypeQualifierTag> DbgTypeQulifierMap;
@@ -1104,6 +1273,15 @@ inline void DbgExpressionOpCodeMap::init() {
   add(dwarf::DW_OP_constu,              SPIRVDebug::Constu);
   add(dwarf::DW_OP_LLVM_fragment,       SPIRVDebug::Fragment);
   add(dwarf::DW_OP_LLVM_convert,        SPIRVDebug::Convert);
+  add(dwarf::DW_OP_addr,                SPIRVDebug::Addr);
+  add(dwarf::DW_OP_const1u,             SPIRVDebug::Const1u);
+  add(dwarf::DW_OP_const1s,             SPIRVDebug::Const1s);
+  add(dwarf::DW_OP_const2u,             SPIRVDebug::Const2u);
+  add(dwarf::DW_OP_const2s,             SPIRVDebug::Const2s);
+  add(dwarf::DW_OP_const4u,             SPIRVDebug::Const4u);
+  add(dwarf::DW_OP_const4s,             SPIRVDebug::Const4s);
+  add(dwarf::DW_OP_const8u,             SPIRVDebug::Const8u);
+  add(dwarf::DW_OP_const8s,             SPIRVDebug::Const8s);
   add(dwarf::DW_OP_consts,              SPIRVDebug::Consts);
   add(dwarf::DW_OP_dup,                 SPIRVDebug::Dup);
   add(dwarf::DW_OP_drop,                SPIRVDebug::Drop);
@@ -1129,6 +1307,7 @@ inline void DbgExpressionOpCodeMap::init() {
   add(dwarf::DW_OP_le,                  SPIRVDebug::Le);
   add(dwarf::DW_OP_lt,                  SPIRVDebug::Lt);
   add(dwarf::DW_OP_ne,                  SPIRVDebug::Ne);
+  add(dwarf::DW_OP_skip,                SPIRVDebug::Skip);
   add(dwarf::DW_OP_lit0,                SPIRVDebug::Lit0);
   add(dwarf::DW_OP_lit1,                SPIRVDebug::Lit1);
   add(dwarf::DW_OP_lit2,                SPIRVDebug::Lit2);
@@ -1227,11 +1406,31 @@ inline void DbgExpressionOpCodeMap::init() {
   add(dwarf::DW_OP_breg31,              SPIRVDebug::Breg31);
   add(dwarf::DW_OP_regx,                SPIRVDebug::Regx);
   add(dwarf::DW_OP_bregx,               SPIRVDebug::Bregx);
+  add(dwarf::DW_OP_piece,               SPIRVDebug::Piece);
   add(dwarf::DW_OP_deref_size,          SPIRVDebug::DerefSize );
   add(dwarf::DW_OP_xderef_size,         SPIRVDebug::XderefSize );
   add(dwarf::DW_OP_nop,                 SPIRVDebug::Nop);
   add(dwarf::DW_OP_push_object_address, SPIRVDebug::PushObjectAddress );
-  add(dwarf::DW_OP_LLVM_arg,            SPIRVDebug::LLVMArg);
+
+
+  add(dwarf::DW_OP_call2,                 SPIRVDebug::Call2);
+  add(dwarf::DW_OP_call4,                 SPIRVDebug::Call4);
+  add(dwarf::DW_OP_call_ref,              SPIRVDebug::CallRef);
+  add(dwarf::DW_OP_form_tls_address,      SPIRVDebug::FormTlsAddress);
+  add(dwarf::DW_OP_call_frame_cfa,        SPIRVDebug::CallFrameCfa);
+  add(dwarf::DW_OP_implicit_value,        SPIRVDebug::ImplicitValue);
+  add(dwarf::DW_OP_implicit_pointer,      SPIRVDebug::ImplicitPointer);
+  add(dwarf::DW_OP_addrx,                 SPIRVDebug::Addrx);
+  add(dwarf::DW_OP_constx,                SPIRVDebug::Constx);
+  add(dwarf::DW_OP_entry_value,           SPIRVDebug::EntryValue);
+  add(dwarf::DW_OP_const_type,            SPIRVDebug::ConstTypeOp);
+  add(dwarf::DW_OP_regval_type,           SPIRVDebug::RegvalType);
+  add(dwarf::DW_OP_deref_type,            SPIRVDebug::DerefType);
+  add(dwarf::DW_OP_xderef_type,           SPIRVDebug::XderefType);
+  add(dwarf::DW_OP_reinterpret,           SPIRVDebug::Reinterpret);
+  add(dwarf::DW_OP_LLVM_arg,              SPIRVDebug::LLVMArg);
+  add(dwarf::DW_OP_LLVM_implicit_pointer, SPIRVDebug::ImplicitPointerTag);
+  add(dwarf::DW_OP_LLVM_tag_offset,       SPIRVDebug::TagOffset);
 }
 
 typedef SPIRVMap<dwarf::Tag, SPIRVDebug::ImportedEntityTag>
@@ -1242,6 +1441,15 @@ inline void DbgImportedEntityMap::init() {
   add(dwarf::DW_TAG_imported_declaration, SPIRVDebug::ImportedDeclaration);
 }
 
+typedef SPIRVMap<llvm::DIFile::ChecksumKind, SPIRVDebug::FileChecksumKind>
+  DbgChecksumKindMap;
+template <>
+inline void DbgChecksumKindMap::init() {
+  add(llvm::DIFile::CSK_MD5,    SPIRVDebug::MD5);
+  add(llvm::DIFile::CSK_SHA1,   SPIRVDebug::SHA1);
+  add(llvm::DIFile::CSK_SHA256, SPIRVDebug::SHA256);
+}
+
 } // namespace SPIRV
 
 #endif // SPIRV_DEBUG_H
